"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable unified-signatures
/**
 * ReplicationWrapper wraps one or more PouchDB.Replication.Syncs and can be used
 * to lazily add more sync results to the middleware after the fact
 * (e.g. starting replication after the user logged in)
 */
var ReplicationWrapper = (function () {
    function ReplicationWrapper() {
        this.wrapped = [];
        this.listeners = {
            change: [],
            active: [],
            complete: [],
            paused: [],
            denied: [],
            error: [],
        };
    }
    ReplicationWrapper.prototype.on = function (event, listener) {
        this.wrapped.forEach(function (sync) {
            sync.on(event, listener);
        });
        this.listeners[event].push(listener);
        return this;
    };
    ReplicationWrapper.prototype.cancel = function () {
        this.wrapped.forEach(function (sync) {
            sync.cancel();
        });
    };
    ReplicationWrapper.prototype.add = function (sync) {
        var _this = this;
        this.wrapped.push(sync);
        var types = ['change', 'active', 'complete', 'paused', 'denied', 'error'];
        types.forEach(function (event) {
            _this.listeners[event].forEach(function (listener) {
                sync.on(event, listener);
            });
        });
        return this;
    };
    return ReplicationWrapper;
}());
exports.ReplicationWrapper = ReplicationWrapper;
//# sourceMappingURL=ReplicationWrapper.js.map