"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var sync = require("../index");
var now = new Date();
var kind = 'kind';
var otherKind = 'something';
var otherModel = {
    kind: otherKind,
    _id: 'nope',
    createdAt: now,
    modifiedAt: now,
};
var emptyState;
var emptyAction;
var state;
var model1 = {
    kind: kind,
    value: 1,
    _id: '1234',
    createdAt: now,
    modifiedAt: now,
};
var model2 = {
    kind: kind,
    value: 2,
    _id: '2345',
    createdAt: now,
    modifiedAt: now,
};
var model3 = {
    kind: kind,
    value: 3,
    _id: '3456',
    createdAt: now,
    modifiedAt: now,
};
var models = [model1, model2];
var reducer;
beforeEach(function () {
    // tslint:disable-next-line no-any
    emptyState = undefined;
    // tslint:disable-next-line no-any
    emptyAction = {};
    reducer = sync.createReducer(kind);
    state = reducer(emptyState, emptyAction);
});
describe('create reducer', function () {
    it('handles initial state', function () {
        expect(state).toMatchObject({ initialized: false, models: [] });
    });
    describe('handles MODEL_INITIALIZED', function () {
        it('regularly', function () {
            state = reducer(state, sync.modelInitialized(kind));
            expect(state).toMatchObject({ initialized: true });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.modelInitialized(otherKind));
            expect(state).toMatchObject({ initialized: false });
        });
    });
    describe('handles LOAD_MODELS', function () {
        it('regularly', function () {
            state = reducer(state, sync.loadModels(models, kind));
            expect(state).toMatchObject({ models: models });
        });
        it('and it overrides the current models', function () {
            state = reducer(state, sync.insertModel(model3));
            state = reducer(state, sync.loadModels(models, kind));
            expect(state).toMatchObject({ models: models });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.loadModels([otherModel], otherKind));
            expect(state).toMatchObject({ models: [] });
        });
    });
    describe('handles INSERT_MODEL', function () {
        it('regularly', function () {
            state = reducer(state, sync.insertModel(model1));
            expect(state).toMatchObject({ models: [model1] });
        });
        it('multiple times', function () {
            state = reducer(state, sync.insertModel(model1));
            state = reducer(state, sync.insertModel(model2));
            expect(state).toMatchObject({ models: [model1, model2] });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.insertModel(otherModel));
            expect(state).toMatchObject({ models: [] });
        });
    });
    describe('handles INSERT_BULK_MODELS', function () {
        it('regularly', function () {
            state = reducer(state, sync.insertBulkModels(models, kind));
            expect(state).toMatchObject({ models: models });
        });
        it('multiple times', function () {
            state = reducer(state, sync.insertBulkModels(models, kind));
            state = reducer(state, sync.insertBulkModels([model3], kind));
            expect(state).toMatchObject({ models: [model1, model2, model3] });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.insertBulkModels([otherModel], otherKind));
            expect(state).toMatchObject({ models: [] });
        });
    });
    describe('handles UPDATE_MODEL', function () {
        it('regularly', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateModel(__assign({}, model1, { value: 2 })));
            expect(state).toMatchObject({
                models: [__assign({}, model1, { value: 2 }), model2],
            });
        });
        it('multiple times', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateModel(__assign({}, model1, { value: 2 })));
            state = reducer(state, sync.updateModel(__assign({}, model1, { value: 3 })));
            expect(state).toMatchObject({
                models: [__assign({}, model1, { value: 3 }), model2],
            });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateModel(otherModel));
            expect(state).toMatchObject({ models: models });
        });
    });
    describe('handles UPDATE_BULK_MODELS', function () {
        it('regularly', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateBulkModels([__assign({}, model1, { value: 2 })], kind));
            expect(state).toMatchObject({
                models: [__assign({}, model1, { value: 2 }), model2],
            });
        });
        it('multiple times', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateBulkModels([__assign({}, model1, { value: 2 })], kind));
            state = reducer(state, sync.updateBulkModels([__assign({}, model1, { value: 3 })], kind));
            expect(state).toMatchObject({
                models: [__assign({}, model1, { value: 3 }), model2],
            });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.updateBulkModels([otherModel], otherKind));
            expect(state).toMatchObject({ models: models });
        });
    });
    describe('handles REMOVE_MODEL', function () {
        it('regularly', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeModel({ _id: model1._id, _rev: model1._rev }, kind));
            expect(state).toMatchObject({
                models: [model2],
            });
        });
        it('multiple times', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeModel({ _id: model1._id, _rev: model1._rev }, kind));
            state = reducer(state, sync.removeModel({ _id: model2._id, _rev: model2._rev }, kind));
            expect(state).toMatchObject({
                models: [],
            });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeModel({ _id: otherModel._id, _rev: otherModel._rev }, otherKind));
            expect(state).toMatchObject({ models: models });
        });
    });
    describe('handles REMOVE_BULK_MODELS', function () {
        it('regularly', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeBulkModels([{ _id: model1._id, _rev: model1._rev }], kind));
            expect(state).toMatchObject({
                models: [model2],
            });
        });
        it('multiple times', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeBulkModels([{ _id: model1._id, _rev: model1._rev }], kind));
            state = reducer(state, sync.removeBulkModels([{ _id: model2._id, _rev: model2._rev }], kind));
            expect(state).toMatchObject({
                models: [],
            });
        });
        it('for other kinds', function () {
            state = reducer(state, sync.loadModels(models, kind));
            state = reducer(state, sync.removeBulkModels([{ _id: otherModel._id, _rev: otherModel._rev }], otherKind));
            expect(state).toMatchObject({ models: models });
        });
    });
});
//# sourceMappingURL=reducer.spec.js.map