"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var constants = require("./constants");
var uuid_1 = require("./utils/uuid");
function initialized(name) {
    return {
        type: constants.INITIALIZED,
        meta: {
            name: name,
        },
    };
}
exports.initialized = initialized;
function modelInitialized(kind) {
    return { type: constants.MODEL_INITIALIZED, meta: { kind: kind } };
}
exports.modelInitialized = modelInitialized;
function loadModels(models, kind) {
    return { type: constants.LOAD_MODELS, payload: models, meta: { kind: kind } };
}
exports.loadModels = loadModels;
function insertModel(model, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.INSERT_MODEL,
        payload: model,
        meta: { kind: model.kind, fromSync: fromSync },
    };
}
exports.insertModel = insertModel;
function insertBulkModels(models, kind, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.INSERT_BULK_MODELS,
        payload: models,
        meta: { kind: kind, fromSync: fromSync },
    };
}
exports.insertBulkModels = insertBulkModels;
function updateModel(model, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.UPDATE_MODEL,
        payload: model,
        meta: { kind: model.kind, fromSync: fromSync },
    };
}
exports.updateModel = updateModel;
function updateBulkModels(models, kind, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.UPDATE_BULK_MODELS,
        payload: models,
        meta: { kind: kind, fromSync: fromSync },
    };
}
exports.updateBulkModels = updateBulkModels;
function removeModel(doc, kind, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.REMOVE_MODEL,
        payload: doc,
        meta: { kind: kind, fromSync: fromSync },
    };
}
exports.removeModel = removeModel;
function removeBulkModels(models, kind, fromSync) {
    if (fromSync === void 0) { fromSync = false; }
    return {
        type: constants.REMOVE_BULK_MODELS,
        payload: models,
        meta: { kind: kind, fromSync: fromSync },
    };
}
exports.removeBulkModels = removeBulkModels;
function modelError(error, operation) {
    return {
        type: constants.MODEL_ERROR,
        payload: error,
        meta: { operation: operation },
    };
}
exports.modelError = modelError;
/**
 * SyncModelBase offers a basic implementation of setting/creating _id, createdAt and modifiedAt
 */
var SyncModelBase = (function () {
    function SyncModelBase(data) {
        this.kind = data.kind;
        this._id = data._id !== undefined ? data._id : uuid_1.uuid();
        this._rev = data._rev;
        this.createdAt = data.createdAt !== undefined ? data.createdAt : new Date();
        this.modifiedAt =
            data.modifiedAt !== undefined ? data.modifiedAt : new Date();
    }
    return SyncModelBase;
}());
exports.SyncModelBase = SyncModelBase;
function isDeletedDoc(doc) {
    return '_deleted' in doc;
}
exports.isDeletedDoc = isDeletedDoc;
function isModel(doc) {
    return doc !== undefined && 'kind' in doc;
}
exports.isModel = isModel;
function isModelToSync(doc, modelsToSync) {
    if (modelsToSync === void 0) { modelsToSync = []; }
    return isModel(doc) && modelsToSync.indexOf(doc.kind) >= 0;
}
exports.isModelToSync = isModelToSync;
function isAction(action) {
    return typeof action !== 'function' && 'type' in action;
}
exports.isAction = isAction;
function isInsertAction(action) {
    return isAction(action) && action.type === constants.INSERT_MODEL;
}
exports.isInsertAction = isInsertAction;
function isBulkInsertAction(action) {
    return isAction(action) && action.type === constants.INSERT_BULK_MODELS;
}
exports.isBulkInsertAction = isBulkInsertAction;
function isUpdateAction(action) {
    return isAction(action) && action.type === constants.UPDATE_MODEL;
}
exports.isUpdateAction = isUpdateAction;
function isBulkUpdateAction(action) {
    return isAction(action) && action.type === constants.UPDATE_BULK_MODELS;
}
exports.isBulkUpdateAction = isBulkUpdateAction;
function isRemoveAction(action) {
    return isAction(action) && action.type === constants.REMOVE_MODEL;
}
exports.isRemoveAction = isRemoveAction;
function isBulkRemoveAction(action) {
    return isAction(action) && action.type === constants.REMOVE_BULK_MODELS;
}
exports.isBulkRemoveAction = isBulkRemoveAction;
function hasMeta(action) {
    return 'meta' in action;
}
exports.hasMeta = hasMeta;
function hasFromSyncMeta(action) {
    return 'fromSync' in action.meta;
}
function isFromSync(action) {
    if (!isAction(action)) {
        return false;
    }
    if (!hasMeta(action)) {
        return false;
    }
    if (!hasFromSyncMeta(action)) {
        return false;
    }
    return action.meta.fromSync;
}
exports.isFromSync = isFromSync;
//# sourceMappingURL=actions.js.map