"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
// tslint:disable max-func-body-length
function createReducer(kind) {
    var initialState = { models: [], initialized: false };
    return function (state, action) {
        if (state === void 0) { state = initialState; }
        switch (action.type) {
            case index_1.MODEL_INITIALIZED:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { initialized: true });
            case index_1.LOAD_MODELS:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { models: action.payload });
            case index_1.INSERT_MODEL:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { models: state.models.concat([action.payload]) });
            case index_1.INSERT_BULK_MODELS:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { models: state.models.concat(action.payload) });
            case index_1.UPDATE_MODEL:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { models: state.models.map(function (m) { return (m._id === action.payload._id ? action.payload : m); }) });
            case index_1.UPDATE_BULK_MODELS: {
                if (action.meta.kind !== kind) {
                    return state;
                }
                var ids_1 = action.payload.map(function (m) { return m._id; });
                return __assign({}, state, { models: state.models.map(function (m) {
                        if (ids_1.indexOf(m._id) === -1) {
                            return m;
                        }
                        return action.payload[ids_1.indexOf(m._id)];
                    }) });
            }
            case index_1.REMOVE_MODEL:
                if (action.meta.kind !== kind) {
                    return state;
                }
                return __assign({}, state, { models: state.models.filter(function (m) { return m._id !== action.payload._id; }) });
            case index_1.REMOVE_BULK_MODELS: {
                if (action.meta.kind !== kind) {
                    return state;
                }
                var ids_2 = action.payload.map(function (m) { return m._id; });
                return __assign({}, state, { models: state.models.filter(function (m) { return ids_2.indexOf(m._id) === -1; }) });
            }
            default:
                return state;
        }
    };
}
exports.createReducer = createReducer;
//# sourceMappingURL=reducer.js.map